#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "scriptread.hh"
#include "gfx.hh"
#include "font.hh"
#define BUFLEN 100
/*******************************************************************************
**readscript(const char *file)
*******************************************************************************/
int script::readscript(const char *file) {
	ifstream scriptopen (file);
	if( !scriptopen.is_open() ) {
		cout << "The file was not opened\n";
		exit(1);
	}
	char buffer[BUFLEN];
	/***************************************************************************
	** exec loop
	***************************************************************************/
	linecount=1;
	gfx *pgfx = gfx::instance(); //get gfx pointer
	while(!scriptopen.eof()) { //the main exec loop
		scriptopen.getline(buffer,BUFLEN); //get a new line
		if( getcmd(&buffer[0]) ) {
			cout << "Error parsing cmd at line: " << linecount <<endl;
			exit(1);
		}
		if(!cmd[0]=='\0') { //the line is not a comment

			if( getargs(&buffer[0]) ) {
				cout << "Error parsing arguments at line: " << linecount <<endl;
				exit(1);
			}
			//exec the cmd
			if(!checkstring(cmd,"text\0")) {
				printtext();
			}
			else if( !checkstring(cmd,"setimg\0") ) {
				setimg(args);
			}
			else if( !checkstring(cmd,"hop\0") ) {
				hop();
			}
			else if( !checkstring(cmd,"bounce\0") ) {
				bounce();
			}
			else if( !checkstring(cmd, "sleep\0") ) {
				sleep(atoi(args));
			}
			else if ( !checkstring(cmd, "usleep\0") ) {
				usleep(atoi(args));
			}
			else if( !checkstring(cmd,"clearscreen\0") ) {
				clearscreen();
			}
			//unknown cmd
			else {
				cout << "Unknown command at line: " << linecount << " exiting now..." <<endl;
				exit(1);
			}
		}
		linecount++;
		pgfx->update();
	}
	scriptopen.close();
	return 0;
}
/*******************************************************************************
* Script execution funcitons
*******************************************************************************/
int script::setimg(char *file) {
	int retval = 0;
	gfx *pgfx = gfx::instance(); //get a pointer to gfx (that should have been created in main()
	MEM_Sprite img;
	int xcord = (WIDTH - img.width) / 2;
	if(!pgfx->loadPCX(&img,file)) {
		int xcord = (WIDTH - img.width) / 2;
		pgfx->drawMEM(&img,xcord,0);
		delete img.data;
	}
	else {
		retval = 1;
	}
	return retval;
}

int script::printtext() { //this function does no errorchecking, but putint doesn't write outside of screen
	int retval=0;
	font *pfont=font::instance();
	pfont->write(args,-1,HEIGHT-pfont->height); //-1 to center
	return retval;
}

int script::bounce() {
	cout << "now i bounce\n";
	return 0;
}

int script::hop() {
	cout << "Now i hop\n";
	return 0;
}

int script::clearscreen() {
	int r,g,b;
	r = atoi(&args[0]);
	int i=0;
	while(args[i]!=',' && i < 100) {
		i++;
	}
	i++;
	g = atoi(&args[i]);
	while(args[i]!=',' && i < 100) {
		i++;
	}
	i++;
	b = atoi(&args[i]);
	gfx *pgfx = gfx::instance();
	pgfx->clearscreen(r,g,b);
	return 0;
}

int script::getcmd(char *buffer) { //puts the current cmd in script.cmd. puts null if comment
	int retval=0;
	int i=0;
	if(buffer[0]==';') {
		cmd[0]='\0';
	}
	else {
		while(1) {
			if(buffer[i]=='\0') {
				retval=1;
				break;
			}
			if(buffer[i]=='(') {
				strncpy(&cmd[0],buffer,i);
				cmd[i]='\0';
				break;
			}
			i++;
		}
	}
	return retval;
}

int script::getargs(char *buffer) {
	int retval=0;
	int i=0;
	while(1) { //i do the break;ing myself
		if(buffer[i]=='(') {
			break;
		}
		if(buffer[i]=='\0') { //unparsable
			retval=1;
			break;
		}
		i++;
	}

	if(!retval) { //if not error, do the rest
		i++; //i is right after '('
		char *temp=&buffer[i];
		i=0;
		while(1) { //scan for ')' in temp.
			if(temp[i]==')') {
				break;
			}
			if(temp[i]=='\0') {
				retval=1; //the script was unparsable
				break;
			}
			i++;
		}
		if(!retval) {
			strncpy(args,temp,i);
			args[i]='\0';
		}
	}
	return retval;
}
/*******************************************************************************
* script::checkstring(const char *one, const char *two)
* used to check if two strings match. This could most likley be done by some library
* function
* 2002-04-15 changed to return 1 for strings that are identical to the end of
* the first string but are realy different.
*******************************************************************************/

int script::checkstring(const char *one, const char *two) { //returns 0 if identical strings
	int i=0;
	int retval=0;
	while(one[i]!='\0' && two[i]!='\0') {
		if(one[i+1]=='\0' && two[i+1]!='\0') { //if next char in one is null and next in two isn't
			retval=1;
			break;
		}
		if(two[i+1]=='\0' && one[i+1]!='\0') { //other way 'round =)
			retval=1;
			break;
		}
		if(one[i]!=two[i]) {
			retval=1;
			break;
		}
		i++;
	}
	return retval;
}
