/*
 *
 * TinyPTC SDL v0.2.1 ported by Alessandro Gatti (a.gatti@tiscalinet.it)
 * Original Windows version by Glenn Fiedler (ptc@gaffer.org)
 * http://www.gaffer.org/tinyptc
 *
 */

#include <stdlib.h>
#include <SDL/SDL.h>
#include "tinyptc.h"

#define PTC_ALPHA_MASK 0xFF000000
#define PTC_RED_MASK   0x00FF0000
#define PTC_GREEN_MASK 0x0000FF00
#define PTC_BLUE_MASK  0x000000FF
#define PTC_WINDOWED true

static int ptc_stored_width;
static int ptc_stored_height;
static SDL_Surface *ptc_video_surface;
static SDL_Surface *ptc_blit_surface;
static SDL_Surface *ptc_buffer_surface;
static SDL_Event ptc_sdl_event;
static int32 *ptc_buffer;

#ifndef __PTC_WINDOWED__
static int ptc_pitch;
#endif /* !__PTC_WINDOWED__ */

int
ptc_open (char *title, int width, int height)
{
	if (SDL_Init (SDL_INIT_VIDEO) < 0) {
		return 0;
	}
	atexit (SDL_Quit);
#ifdef __PTC_WINDOWED__
	SDL_WM_SetCaption (title, NULL);
#endif /* __PTC_WINDOWED__ */
	ptc_video_surface=SDL_SetVideoMode (width, height,
#ifdef __PTC_ENABLE_CONVERSIONS__
	                                    32,
#else
	                                    0,
#endif /* __PTC_ENABLE_CONVERSIONS__ */
	                                    SDL_HWSURFACE | SDL_DOUBLEBUF
#ifndef __PTC_WINDOWED__
	                                    | SDL_FULLSCREEN
#endif /* !__PTC_WINDOWED__ */
	                                    );
	if (ptc_video_surface == NULL) {
		return 0;
	}
	SDL_ShowCursor(0);
	ptc_stored_width=width;
	ptc_stored_height=height;
#ifndef __PTC_WINDOWED__
	ptc_pitch=ptc_video_surface->pitch;
#endif /* !__PTC_WINDOWED__ */
	ptc_buffer=(int32 *)calloc(ptc_stored_width*ptc_stored_height, sizeof(int32));
	if (ptc_buffer==NULL) {
		return 0;
	}
	ptc_buffer_surface=SDL_CreateRGBSurfaceFrom(ptc_buffer, ptc_stored_width, ptc_stored_height, 32, ptc_stored_width*sizeof(int), PTC_RED_MASK, PTC_GREEN_MASK, PTC_BLUE_MASK, PTC_ALPHA_MASK);
#ifndef __PTC_ENABLE_CONVERSIONS__
	ptc_blit_surface=ptc_buffer_surface;
#endif /* !__PTC_ENABLE_CONVERSIONS__ */
	return 1;
}

int
ptc_update (void *buffer)
{
	int ptc_return_code;
#ifndef __PTC_WINDOWED__
	unsigned int ptc_counter;
	unsigned int ptc_index;
#endif /* __PTC_WINDOWED__ */

	memcpy(ptc_buffer_surface->pixels, buffer, ptc_stored_width*ptc_stored_height*sizeof(int));
#ifdef __PTC_ENABLE_CONVERSIONS__
	ptc_blit_surface=SDL_DisplayFormat(ptc_buffer_surface);
	if (ptc_blit_surface==NULL) {
		return 0;
	}
#else
	ptc_blit_surface=ptc_buffer_surface;
#endif /* __PTC_ENABLE_CONVERSIONS__ */
	ptc_return_code=SDL_BlitSurface(ptc_blit_surface, NULL, ptc_video_surface, NULL);
	if (ptc_return_code!=0) {
		return 0;
	}
#ifdef __PTC_ENABLE_CONVERSIONS__
	SDL_FreeSurface(ptc_blit_surface);
#endif /* __PTC_ENABLE_CONVERSIONS__ */
	ptc_return_code=SDL_Flip(ptc_video_surface);
	if (ptc_return_code!=0) {
		return 0;
	}
	ptc_process_events();
	return 1;
}

int
ptc_process_events (void)
{
	char *ptc_keypressed;

	if (SDL_PollEvent (&ptc_sdl_event)) {
		switch (ptc_sdl_event.type) {
		case SDL_KEYDOWN: {
			ptc_keypressed=SDL_GetKeyState(NULL);
			if (ptc_keypressed[SDLK_ESCAPE]==SDL_PRESSED) {
				ptc_close();
				exit(0);
			}
		}; break;
		case SDL_QUIT: {
			ptc_close();
			exit(0);
		}; break;
		}
	}
	return 0;
}

void
ptc_close (void)
{
	SDL_ShowCursor(1);
	SDL_FreeSurface(ptc_buffer_surface);
	SDL_FreeSurface(ptc_video_surface);
	free(ptc_buffer);
}
